//#extension GL_OES_EGL_image_external : require
precision mediump float;

uniform int FaceAmount;
uniform vec2 FrameSize;

uniform vec2 FaceAngle0;
uniform vec2 FaceAngle1;
uniform vec2 FaceAngle2;
uniform vec2 FaceAngle3;
uniform vec2 FaceAngle4;

uniform vec2 FaceAlign0;
uniform vec2 FaceAlign1;
uniform vec2 FaceAlign2;
uniform vec2 FaceAlign3;
uniform vec2 FaceAlign4;

uniform vec4 StickerInfo0;
uniform vec4 StickerInfo1;
uniform vec4 StickerInfo2;
uniform vec4 StickerInfo3;
uniform vec4 StickerInfo4;

varying vec2 map_coord;
uniform sampler2D map_texture;

vec2 CalcRealNormCoord( vec2 Angle,vec2 Align, vec4 stickerInfo, vec2 frameCoord ){
    vec2 transCoord = vec2(0.0, 0.0);           /* 逆旋转后坐标 */
    vec2 rotatedAlign = vec2(0.0, 0.0);         /* 绑定点坐标，与上一步相同，进行旋转计算 */

    // 逆旋转（720.0, 1280.0）
    // angle.x : sin(a), angle.y : cos(a)
    // 参考http://blog.chinaunix.net/uid-20306372-id-1707267.html
    // 博客中逆时针，此处顺时针, ( x' y' 反求 x y , 以(0,0)为中心 )
    transCoord.x = Angle.x * frameCoord.x + Angle.y * frameCoord.y;
    transCoord.y = Angle.x * frameCoord.y - Angle.y * frameCoord.x;

    // 上面计算的是 Sticker 中各像素点绕 (0,0) 旋转得到的坐标，
    // 这里计算的是 Sticker 中绑定点旋转后的坐标，用于 in_TransformedCoord 偏移
    rotatedAlign.x = Angle.x * Align.x + Angle.y * Align.y;
    rotatedAlign.y = Angle.x * Align.y - Angle.y * Align.x;

    transCoord = transCoord + stickerInfo.zw - rotatedAlign;

    vec2 normCoord = vec2( transCoord.x / stickerInfo.x, transCoord.y / stickerInfo.y );

    return normCoord;
}

void main() {
    gl_FragColor = vec4(0.0,0.0,0.0,0.0);

    // 帧宽高
    float in_FrameW = FrameSize.x;
    float in_FrameH = FrameSize.y;

    vec2 in_NormCoord = vec2(0.0, 0.0);
    /* 将归一化坐标转化为像素数据 */
    vec2 in_FrameCoord;
    /* 临时纹理像素 */
    vec4 in_TextureColor;
    float in_Alpha = 0.0;
    in_TextureColor = vec4(0.0, 0.0, 0.0, 0.0);
    in_FrameCoord = vec2( map_coord.x * in_FrameW, map_coord.y * in_FrameH );

    if(FaceAmount>0)
    {
        in_NormCoord = CalcRealNormCoord( FaceAngle0, FaceAlign0, StickerInfo0,in_FrameCoord );
        if (in_NormCoord.x > 0.0 && in_NormCoord.y > 0.0 && in_NormCoord.x < 1.0 && in_NormCoord.y < 1.0)
        {
            in_TextureColor = texture2D(map_texture, in_NormCoord);
            in_Alpha = in_TextureColor.w;
            gl_FragColor = gl_FragColor * ( 1.0 - in_Alpha ) + in_TextureColor * in_Alpha;
        }
    }

    if(FaceAmount>1)
    {
        in_NormCoord = CalcRealNormCoord( FaceAngle1, FaceAlign1, StickerInfo1,in_FrameCoord );
        if (in_NormCoord.x > 0.0 && in_NormCoord.y > 0.0 && in_NormCoord.x < 1.0 && in_NormCoord.y < 1.0)
        {
            in_TextureColor = texture2D(map_texture, in_NormCoord);
            in_Alpha = in_TextureColor.w;
            gl_FragColor = gl_FragColor * ( 1.0 - in_Alpha ) + in_TextureColor * in_Alpha;
        }
    }

    if(FaceAmount>2)
    {
        in_NormCoord = CalcRealNormCoord( FaceAngle2, FaceAlign2, StickerInfo2,in_FrameCoord );
        if (in_NormCoord.x > 0.0 && in_NormCoord.y > 0.0 && in_NormCoord.x < 1.0 && in_NormCoord.y < 1.0)
        {
            in_TextureColor = texture2D(map_texture, in_NormCoord);
            in_Alpha = in_TextureColor.w;
            gl_FragColor = gl_FragColor * ( 1.0 - in_Alpha ) + in_TextureColor * in_Alpha;
        }
    }

    if(FaceAmount>3)
    {
        in_NormCoord = CalcRealNormCoord( FaceAngle3, FaceAlign3, StickerInfo3,in_FrameCoord );
        if (in_NormCoord.x > 0.0 && in_NormCoord.y > 0.0 && in_NormCoord.x < 1.0 && in_NormCoord.y < 1.0)
        {
            in_TextureColor = texture2D(map_texture, in_NormCoord);
            in_Alpha = in_TextureColor.w;
            gl_FragColor = gl_FragColor * ( 1.0 - in_Alpha ) + in_TextureColor * in_Alpha;
        }
    }

    if(FaceAmount>4)
    {
        in_NormCoord = CalcRealNormCoord( FaceAngle4, FaceAlign4, StickerInfo4, in_FrameCoord );
        if (in_NormCoord.x > 0.0 && in_NormCoord.y > 0.0 && in_NormCoord.x < 1.0 && in_NormCoord.y < 1.0)
        {
            in_TextureColor = texture2D(map_texture, in_NormCoord);
            in_Alpha = in_TextureColor.w;
            gl_FragColor = gl_FragColor * ( 1.0 - in_Alpha ) + in_TextureColor * in_Alpha;
        }
    }


    //vec4 frame_color = texture2D( map_texture, texFrameCoord );
    //gl_FragColor = frame_color;
}