#extension GL_OES_EGL_image_external : require
precision mediump float;
varying vec2 texFrameCoord;
varying vec2 texMapCoord;

uniform int faceCount;
uniform float faceDetected;
uniform float trans_x;
uniform float trans_y;

uniform vec2 mapSize;
uniform vec2 frameSize;

uniform samplerExternalOES frame_texture;
uniform sampler2D map_texture;

uniform vec2 alignPoint0;
uniform vec2 alignPoint1;
uniform vec2 alignPoint2;
uniform vec2 alignPoint3;
uniform vec2 alignPoint4;

uniform vec2 size0;
uniform vec2 size1;
uniform vec2 size2;
uniform vec2 size3;
uniform vec2 size4;

uniform vec2 angle0;
uniform vec2 angle1;
uniform vec2 angle2;
uniform vec2 angle3;
uniform vec2 angle4;

//ivec2 frameSize = textureSize(frame_texture, 0);
//ivec2 mapSize = textureSize(map_texture, 0);

void main() {

    gl_FragColor = texture2D(frame_texture, texFrameCoord);

    if( faceCount < 1 )
    {
        return;
    }

    float x_a = frameSize.x;
    float y_a = frameSize.y;

    vec2 rotateCoord = vec2(0.0,0.0);
    vec2 scrPix = vec2( texMapCoord.x * x_a, texMapCoord.y * y_a );

    vec2 pngSize = vec2(size0.x * x_a, size0.y * y_a);
    vec2 png_center = pngSize * (0.4,0.5);

    vec2 target_pix = vec2( alignPoint0.x, alignPoint0.y );

    // 逆旋转（720.0, 1280.0）
    // angle.x : sin(a), angle.y : cos(a)
    // 参考http://blog.chinaunix.net/uid-20306372-id-1707267.html
    // 博客中逆时针，此处顺时针, ( x' y' 反求 x y , 以(0,0)为中心 )
    rotateCoord.x = angle0.y * scrPix.x + angle0.x * scrPix.y;
    rotateCoord.y = angle0.y * scrPix.y - angle0.x * scrPix.x;

    // 绑定点坐标与上一步相同，进行旋转计算
    vec2 weiyi = vec2(0.0,0.0);
    weiyi.x = angle0.y * target_pix.x + angle0.x * target_pix.y;
    weiyi.y = angle0.y * target_pix.y - angle0.x * target_pix.x;

    rotateCoord = rotateCoord + png_center - weiyi;
    vec4 c1 = vec4(0.0);
    // 缩放
    vec2 realCoord = vec2(rotateCoord.x/pngSize.x, rotateCoord.y/pngSize.y);

    if (realCoord.x > 0.0 && realCoord.y > 0.0 && realCoord.x < 1.0 && realCoord.y < 1.0)
    {
        c1 = texture2D(map_texture, realCoord);
        float alpha_cc = c1.w;
        gl_FragColor = c1 * alpha_cc +  gl_FragColor * (1.0-alpha_cc);
    }else
    {
        gl_FragColor = gl_FragColor;
    }

}